"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const react = require("react");
const reactStore = require("@tanstack/react-store");
const lodashEs = require("lodash-es");
const utils = require("./utils.cjs");
const encoding = require("./encoding.cjs");
const useApi = require("./use-api.cjs");
const api = require("./api.cjs");
const create = () => {
  let isInitialized = false;
  let paramsStore = null;
  let api$1 = null;
  const getLatestParams = (api2) => {
    const params = new URLSearchParams(api2.getSearch());
    const paramsMap = {};
    for (const [key, value] of params) {
      paramsMap[key] = value;
    }
    return paramsMap;
  };
  const init = (contextApi, isClient2) => {
    if (isClient2 && isInitialized) {
      return;
    }
    isInitialized = true;
    api$1 = api.withBatch(contextApi ?? (isClient2 ? api.defaultApi() : api.dummyApi));
    if (isClient2) {
      api$1.registerListener((state) => {
        if (utils.isParamsTransition(state)) {
          return;
        }
        paramsStore.setState(() => getLatestParams(api$1));
      });
      paramsStore = new reactStore.Store(getLatestParams(api$1));
    }
  };
  const decodeWithDefault = (value, defaultValue, {
    decode,
    onError,
    validate
  }) => {
    if (value === void 0) {
      return defaultValue;
    }
    const internalOnError = (decoded) => {
      const onErrorValue = onError == null ? void 0 : onError(value, decoded);
      return onErrorValue ?? defaultValue;
    };
    try {
      const decoded = encoding.decodeParam(value, decode);
      const isValid = validate ? validate(decoded) : true;
      if (isValid) {
        return decoded;
      } else {
        return internalOnError(decoded);
      }
    } catch {
      return internalOnError();
    }
  };
  const useDefaultValue = ({ defaultValue }) => {
    const internalDefaultValue = react.useMemo(() => {
      return utils.getValue(defaultValue);
    }, [defaultValue]);
    return utils.useSmartValue(internalDefaultValue);
  };
  const useParamGet = (paramName, options) => {
    const contextApi = useApi.useContextApi();
    init(contextApi, utils.isClient);
    const internalDefaultValue = useDefaultValue(options);
    const value = utils.isClient ? reactStore.useStore(paramsStore, (s) => s[paramName]) : getLatestParams(api$1)[paramName];
    const decodedValue = react.useMemo(() => decodeWithDefault(value, internalDefaultValue, options), [value, internalDefaultValue, options]);
    return utils.useSmartValue(decodedValue);
  };
  const useParamSet = (paramName, options) => {
    const contextApi = useApi.useContextApi();
    init(contextApi, utils.isClient);
    const internalDefaultValue = useDefaultValue(options);
    return react.useCallback((value, state) => {
      if (!utils.isClient) {
        throw new Error("Cannot use react params on the server");
      }
      const { updateType = "replaceIn", encode } = options;
      const currentValue = paramsStore.state[paramName];
      const internalValue = typeof value === "function" ? value(decodeWithDefault(currentValue, internalDefaultValue, options)) : value;
      const newValue = encoding.encodeParam(internalValue, encode);
      if (currentValue === newValue || currentValue === void 0 && lodashEs.isEqual(internalDefaultValue, value)) {
        return;
      }
      paramsStore.setState((s) => {
        return { ...s, [paramName]: newValue };
      });
      const search = api$1.getSearch();
      const clearAll = !updateType.endsWith("In");
      const isReplace = updateType.startsWith("replace");
      const searchParams = !clearAll ? new URLSearchParams(search) : new URLSearchParams();
      if (!lodashEs.isEqual(internalDefaultValue, value)) {
        searchParams.set(paramName, newValue);
      } else {
        searchParams.delete(paramName);
      }
      const newHref = `?${searchParams.toString()}`;
      const internalState = { [utils.paramsTransitioning]: true, ...state };
      if (isReplace) {
        api$1.replaceState(newHref, internalState);
      } else {
        api$1.pushState(newHref, internalState);
      }
    }, [paramName, internalDefaultValue, options]);
  };
  const useParam = (paramName, options) => {
    const value = useParamGet(paramName, options);
    const setter = useParamSet(paramName, options);
    return [value, setter];
  };
  return {
    useParamGet,
    useParamSet,
    useParam,
    paramsStore,
    api: api$1
  };
};
exports.create = create;
//# sourceMappingURL=core.cjs.map
